open Core.Std
open OUnit

let raises_exception ?(exn) f =
  match exn with
  |Some e ->
    (try f (); false
    with e' when e=e' -> true)
  |None   ->
    (try f (); false
    with _ -> true)

let parsed s =
  let lexbuf = Lexing.from_string s in
  let (sch,_) = Parser.main Lexer.token lexbuf in
  sch

let sats = [|
    "P_1";
    "P_2";
    "P_n";
    "~P_n";
    "Q_i";
    "P_1 /\\ P_2";
    "P_1 /\\ ~P_2";
    "~P_1 /\\ P_2";
    "~P_1 /\\ ~P_2";
    "P_1 \\/ P_2";
    "P_1 \\/ ~P_2";
    "~P_1 \\/ P_2";
    "~P_1 \\/ ~P_2";
    "P_n \\/ P_n+1";
    "P_n \\/ ~P_n+1";
    "~P_n \\/ P_n+1";
    "~P_n \\/ ~P_n+1";
    "P_1 /\\ (~P_2 \\/ P_1)";
    "~P_1 /\\ (~P_2 \\/ P_1)";
  |]

let insats = [|
    "P_1 /\\ ~P_1";
    "P_2 /\\ ~P_2";
    "P_n /\\ ~P_n";
    "P_n+1 /\\ ~P_n+1";
    "P_1 /\\ (~P_1 \\/ P_2) /\\ ~P_2";
    "P_1 /\\ (~P_1 \\/ P_n) /\\ ~P_n";
    "P_n /\\ (~P_n \\/ P_2) /\\ ~P_2";
    "P_n /\\ (~P_n \\/ P_n-1) /\\ ~P_n-1";
    "P_n /\\ (~P_n \\/ P_n+1) /\\ ~P_n+1";
    "P_n /\\ (~P_n \\/ P_n+1) /\\ (~P_n+1 \\/ P_n+2) /\\ ~P_n+2";
    "P_1 /\\ (~P_1 \\/ P_n+1) /\\ (~P_n+1 \\/ P_n+2) /\\ ~P_n+2";
    "P_n /\\ (~P_n \\/ P_1) /\\ (~P_1 \\/ P_n+2) /\\ ~P_n+2";
  |]

let parametrized_test strat = 
  "propositional rules" >:::
    [ "satisfiable" >::
      (fun () ->
        Array.iter sats ~f:(fun s -> s @? ((strat <|| parsed) s = Satisfiability.ModelFound))
      );
    "unsatisfiable" >::
      (fun () ->
        Array.iter insats ~f:(fun s -> s @? ((strat <|| parsed) s = Satisfiability.NoModelFound))
      );
    ]

(*
 *let prop_strat sch =
 *  Regstab.prop_rules ~k:Strategy.ident_strat ~applied:false ~schs:[sch] ~cstr:Constraint.top
 *
 *let test = parametrized_test prop_strat
 *)
